#!/usr/bin/env python3
# see man git-shell

import os
import re
import subprocess
import sys

import stagit

SOC_RE = (
    r"^(git-receive-pack|git-upload-pack|git-upload-archive) "
    r"'?/?([a-z0-9_:-]+)(\.git)?/?'?$"
)


def parse_soc():
    soc = re.match(SOC_RE, os.environ['SSH_ORIGINAL_COMMAND'])
    if not soc:
        sys.stderr.write('invalid command\n')
        sys.exit(1)
    cmd, repo, _ = soc.groups()
    return cmd, repo


if __name__ == '__main__':
    user = sys.argv[1]

    if 'SSH_ORIGINAL_COMMAND' not in os.environ:
        for repo in stagit.config.iter_repos():
            if stagit.config.can_ssh(repo, user):
                print(repo)
        sys.exit(0)

    cmd, repo = parse_soc()

    if not stagit.config.can_ssh(repo, user):
        sys.stderr.write('access denied\n')
        sys.exit(1)

    sys.exit(subprocess.call(
        [*cmd.split('-', 1), repo],
        cwd=stagit.REPO_DIR,
        env={
            **os.environ,
            'STAGIT_USER': user,
            'STAGIT_REPO': repo,
        },
    ))
